/* Document preview */

img.preview {
  display: block;
  border: 1px black;
  margin: 0 auto;
}

.document-folder-icon {
  font-size: 32pt;
  margin-right: 18px;
  position: relative;
  top: 11.5px;
  color: dimgray;
}

.preview-container {
  position: relative;

  .preview-nav-button {
    position: absolute;
    top: 0;
    left: 0;
    bottom: 0;
    display: block;
    z-index: 100;
    width: 15%;
    text-align: center;
    color: transparent;
    opacity: 0.9;

    &:hover {
      color: #0c3b3d;
      cursor: pointer;
    }

    .fa {
      display: inline-block;
      position: absolute;
      top: 50%;
    }
  }

  /* .preview-nav */
  .preview-prev .fa {
    left: 50%;
  }

  .preview-next {
    right: 0;
    left: auto;

    .fa {
      right: 50%;
    }
  }
}

.numberCircle {
  border-radius: 50%;
  width: 17px;
  font-size: 13px;
  border: 2px solid yellowgreen;
  position: relative;
  top: -57px;
  color: white;
  left: 34px;
  background: yellowgreen;
  margin-bottom: -97px;
}

.numberCircle span {
  text-align: center;
  line-height: 13px;
  display: block;
}

.gallery-numberCircle {
  border-radius: 50%;
  width: 45px;
  font-size: 23px;
  border: 2px solid yellowgreen;
  position: relative;
  top: -195px;
  color: white;
  left: 195px;
  background: yellowgreen;
  margin-bottom: -97px;
  font-weight: bold;
}

.gallery-numberCircle span {
  text-align: center;
  line-height: 40px;
  display: block;
}

/* folder / documents objects table */
table#objects-table tbody td > img + i, .folder-items li > img + i {
  position: relative;
  left: -14px;
  font-size: 9px;
}

/* My own (ugly) boxes */
.property {
  &-box .nav > li > a {
    display: inline;
    background-color: white;
  }

  &-list {
    padding-left: 14px;
    padding-right: 14px;
    margin-left: 0;
    margin-bottom: 5px;
    list-style: none;

    li {
      margin-top: 5px;
      margin-bottom: 5px;
    }
  }
}

#objects-table_wrapper img {
  border-radius: 0;
}

.document-table-thumbnail {
  > tbody > tr > td {
    background: white !important;
    line-height: 5.5;
    border-top: 1px solid ghostwhite;
  }
}

.document-preview {
  background-position: -6px;
  background-size: 76px;
  padding: 64px 0 0;
  border: 2px solid #fff;
  outline: 1px solid #eee;
  width: 68px;
  height: 38px;
  display: block;
  float: left;
  margin: 3px -34px 3px 3px;
  position: relative;
}

.document-table-gallery {
  > tbody > tr > td {
    background: white !important;
    line-height: 0;
    border-top: 1px solid ghostwhite;
  }
}

.folder-contents-count {
  background: yellowgreen;
  font-weight: bold;
  font-size: 10pt;
  position: relative;
  display: compact;
  width: 19px;
  height: 19px;
  color: white;
  border-radius: 50%;
  left: 17px;
  top: -57px;
  margin-bottom: -100px;

  > .documents-nb {
    position: relative;
    top: -26px;
    left: 6px;
  }
}

.folder-gallery-contents-count {
  background: yellowgreen;
  font-weight: bold;
  font-size: 10pt;
  position: relative;
  display: compact;
  width: 50px;
  height: 50px;
  color: white;
  border-radius: 50%;
  left: 182px;
  top: -201px;
  margin-bottom: -100px;

  > .documents-nb {
    position: relative;
    font-size: 30pt;
    top: -2px;
    left: 14px;
  }
}

.gallery-folder {
  font-size: 158pt;
  margin-right: 10px;
  position: relative;
  top: 4.5px;
  color: dimgray;
  left: 25px;
  margin-bottom: 20px;
}

.document-folder-download {
  border: none;
  background: transparent;
  display: block;
  margin-bottom: -27px;
  margin-top: -26px;
  margin-left: 14px;
  outline: 0;
}

.document-horizontal-menu {
  float: right;
  display: none;
  position: relative;
  top: -99px;
  right: 89px;
  margin-bottom: -35px;
}

.document-thumbnail-menu {
  border-radius: 0;
  position: absolute;
  top: 53px;
  left: -129px;
  margin-bottom: -144px;
}

.document-gallery-infos {
  margin-left: 11px;
  margin-top: -8px;
  padding-bottom: 50px;
  position: relative;
  top: 20px;

  > p {
    color: #797c7e;
    position: relative;
  }

  > p:nth-child(3) {
    color: black;
  }
}

.document-gallery-dropdownbase {
  position: relative;
  float: right;
  top: -103px;
  right: 11px;
}

.document-gallery-dropdown {
  border: none;
  font-size: 20pt;
  color: #797c7e;
  background: transparent;
  position: relative;
  top: 38px;
  right: 8px;
}

.document-gallery-box {
  border: 1px solid silver;
  width: 258px;
  height: 437px;
  padding: 0;
  margin-left: -3px;
  margin-top: 5px;
}

.upload-files-btn {
  display: inline;
  position: relative;
  padding: 8px;
  top: 3px;
  border: none;
  background: yellowgreen;
  color: white;
  width: 4cm;
  border-radius: 3px;
}

.add-folder-btn {
  display: inline;
  position: relative;
  padding: 8px;
  top: 3px;
  border: 1px solid silver;
  background: white;
  width: 4cm;
  color: gray;
  border-radius: 3px;
}

.document-preview:hover {
  -ms-transform: scale(2, 2); /* IE 9 */
  -webkit-transform: scale(2, 2); /* Safari */
  transform: scale(2, 2); /* Standard syntax */
  z-index: 2;
}

/* File Uploads (currently not used) */
.btn.btn-file {
  overflow: hidden;
  position: relative;
  vertical-align: middle;

  > input[type="file"] {
    width: 100%;
    position: absolute;
    left: 0;
    top: 0;
    opacity: 0;
    cursor: pointer;
  }
}

.fileupload {
  margin-bottom: 9px;

  .uneditable-input {
    display: inline-block;
    margin-bottom: 0;
    vertical-align: middle;
  }

  .thumbnail {
    overflow: hidden;
    display: inline-block;
    margin-bottom: 5px;
    vertical-align: middle;
    text-align: center;
  }

  .thumbnail > img {
    display: inline-block;
    vertical-align: middle;
    max-height: 100%;
  }

  .btn {
    vertical-align: middle;
  }

  &-exists &-new, &-new &-exists {
    display: none;
  }

  &-inline &-controls {
    display: inline;
  }

  &-new .input-append .btn-file {
    -webkit-border-radius: 0 3px 3px 0;
    -moz-border-radius: 0 3px 3px 0;
    border-radius: 0 3px 3px 0;
  }
}
