<html>
<body>

<h1>
  Invalid authentication token. </h1>

<p>
  We're sorry but you authentication token is invalid or expired. </p>

<p> You can unsubscribe from notifications by login in <a href="/login/">
  and going to the "Preferences" menu in the top right corner of the screen.</a>
</p>

</body>
</html>
